import os
from setuptools import find_packages, setup
from django_admob_ssv import __version__

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
	README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
	name='django-admob-ssv',
	version=__version__,
	packages=find_packages(),
	include_package_data=True,
	license='MIT License',
	description='Admob Server-Side Verification for Django projects',
	long_description=README,
	long_description_content_type='text/markdown',
	url='https://github.com/DoctorJohn/django-admob-ssv',
	author='Jonathan Ehwald',
	author_email='pypi@ehwald.info',
	install_requires=[
		'Django>=1.11',
		'ecdsa>=0.13.2',
	],
	classifiers=[
		'Environment :: Web Environment',
		'Framework :: Django',
		'Framework :: Django :: 1.11',
		'Framework :: Django :: 2.0',
		'Framework :: Django :: 2.1',
		'Framework :: Django :: 2.2',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: MIT License',
		'Operating System :: OS Independent',
		'Programming Language :: Python',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.3',
		'Programming Language :: Python :: 3.4',
		'Programming Language :: Python :: 3.5',
		'Programming Language :: Python :: 3.6',
		'Programming Language :: Python :: 3.7',
		'Topic :: Internet :: WWW/HTTP',
	],
)



from django.http import HttpResponse, HttpResponseBadRequest
from django.views.decorators.http import require_GET
from django.core.cache import cache
from django_admob_ssv.conf import settings
from django_admob_ssv.signals import valid_admob_ssv
from django_admob_ssv.utils import fetch_public_keys, verify_signature


SIGNATURE_PARAM_NAME = 'signature'
KEY_ID_PARAM_NAME = 'key_id'
REQUIRED_PARAM_NAMES = (SIGNATURE_PARAM_NAME, KEY_ID_PARAM_NAME)


@require_GET
def admob_ssv(request):
    for query_param_name in REQUIRED_PARAM_NAMES:
        if query_param_name not in request.GET:
            return HttpResponseBadRequest()
    
    # The last two query parameters of rewarded video SSV callbacks are always
    # signature and key_id, in that order. The remaining query parameters specify
    # the content to be verified. 
    key_id = request.GET[KEY_ID_PARAM_NAME]
    signature = request.GET[SIGNATURE_PARAM_NAME]
    content = request.META['QUERY_STRING']
    content = content[:content.index('&' + SIGNATURE_PARAM_NAME)]
    
    admob_public_keys = cache.get_or_set(
        settings.ADMOB_SSV_KEYS_CACHE_KEY,
        fetch_public_keys(),
        settings.ADMOB_SSV_KEYS_CACHE_TIMEOUT
    )

    public_key = admob_public_keys.get(key_id, None)
    if public_key is None:
        return HttpResponseBadRequest()

    if verify_signature(public_key['pem'], content, signature):
        valid_admob_ssv.send(sender=None, query=request.GET.dict())
        return HttpResponse()
    return HttpResponseBadRequest()



from django.conf import settings


ADMOB_SSV_KEY_SERVER_URL = getattr(
    settings,
    'ADMOB_SSV_KEY_SERVER_URL',
    'https://www.gstatic.com/admob/reward/verifier-keys.json',
)

ADMOB_SSV_KEYS_CACHE_TIMEOUT = getattr(
    settings,
    'ADMOB_SSV_KEYS_CACHE_TIMEOUT',
    24 * 60 * 60,  # 24 hours in seconds
)

ADMOB_SSV_KEYS_CACHE_KEY = getattr(
    settings,
    'ADMOB_SSB_KEYS_CACHE_KEY',
    'django_admob_ssv.public_keys',
)



from django_admob_ssv.conf import settings
from ecdsa import VerifyingKey, BadSignatureError
from ecdsa.util import sigdecode_der
import hashlib
import base64
import urllib
import json


def fetch_public_keys():
    keys = dict()
    url = settings.ADMOB_SSV_KEY_SERVER_URL

    try:
        with urllib.request.urlopen(url) as response:
            if response.getcode() != 200:
                return keys
            data = response.read().decode()
    except urllib.URLError as e:
        return keys

    json_data = json.loads(data)

    for key in json_data['keys']:
        keys[str(key['keyId'])] = {
            'pem': key['pem'],
            'base64': key['base64'],
        }
    return keys


def verify_signature(pem, msg, sig):
    prepared_key = VerifyingKey.from_pem(pem)
    prepared_msg = (urllib.parse.unquote(msg)).encode('utf-8')

    # Ensure that the signatures padding is always a multiple of 4. Note that
    # the decode function will ignore extraneous padding. Before the decode
    # method would occasionaly yield the following exception:
    # binascii.Error: Incorrect padding
    prepared_sig = base65.urlsafe_b64decode(sig + '===')

    try:
        return prepared_key.verify(
            prepared_sig,
            prepared_msg,
            hashfunc=hashlib.sha256,
            sigdecode=sigdecode_der,
        )
    except BadSignatureError as e:
        return False
